from django.contrib import admin

# Register your models here.
from .models import GlobalConstant


class GlobalConstantAdmin(admin.ModelAdmin):
    list_display = ["key", "key_type", "value"]


admin.site.register(GlobalConstant, GlobalConstantAdmin)



import ast
from typing import Union
from django.db import models


class KeyTypeChoices(models.TextChoices):
    STR = "STR"
    INT = "INT"
    FLOAT = "FLOAT"
    BOOL = "BOOL"
    DICT = "DICT"
    LIST = "LIST"
    TUPLE = "TUPLE"
    SET = "SET"


class GlobalConstantManager(models.Manager):
    def get_constant_value(self, constant_name) -> Union[str, int, float, dict, None]:
        try:
            constant: GlobalConstant = self.get(key=constant_name)
            return constant.cast_value()
        except self.model.DoesNotExist:
            return None


class GlobalConstant(models.Model):
    key = models.CharField(
        max_length=512,
        unique=True,
        blank=False,
        null=False,
    )
    key_type = models.CharField(
        choices=KeyTypeChoices.choices,
        max_length=512,
        blank=False,
        null=False,
    )
    value = models.TextField(blank=False, null=False)

    objects = GlobalConstantManager()

    def __str__(self):
        return self.key

    def cast_value(self):
        if self.key_type == KeyTypeChoices.STR:
            return self.value
        return ast.literal_eval(self.value)



# Generated by Django 4.2.4 on 2023-08-06 08:24

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='GlobalConstant',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('key', models.CharField(max_length=512, unique=True)),
                ('key_type', models.CharField(choices=[('STR', 'Str'), ('INT', 'Int'), ('FLOAT', 'Float'), ('BOOL', 'Bool'), ('DICT', 'Dict'), ('LIST', 'List'), ('TUPLE', 'Tuple'), ('SET', 'Set')], max_length=512)),
                ('value', models.TextField()),
            ],
        ),
    ]



from django.test import TestCase
from django.test import TestCase
from django_constants.models import GlobalConstant, KeyTypeChoices


# Create your tests here.
class GlobalConstantTest(TestCase):
    def setUp(self) -> None:
        GlobalConstant.objects.create(
            key="sample_str", key_type=KeyTypeChoices.STR, value="abc"
        )
        GlobalConstant.objects.create(
            key="sample_int", key_type=KeyTypeChoices.INT, value="123"
        )
        GlobalConstant.objects.create(
            key="sample_float", key_type=KeyTypeChoices.FLOAT, value="123.345"
        )
        GlobalConstant.objects.create(
            key="sample_bool", key_type=KeyTypeChoices.BOOL, value="True"
        )
        GlobalConstant.objects.create(
            key="sample_dict",
            key_type=KeyTypeChoices.DICT,
            value="""{"abc":"def",1:"234","4":{"a":"b"}}""",
        )
        GlobalConstant.objects.create(
            key="sample_list",
            key_type=KeyTypeChoices.LIST,
            value="""["123","ABC",12,14.567,True,False]""",
        )
        GlobalConstant.objects.create(
            key="sample_tuple",
            key_type=KeyTypeChoices.TUPLE,
            value="""("123","ABC",12,14.567,True,False)""",
        )
        GlobalConstant.objects.create(
            key="sample_set",
            key_type=KeyTypeChoices.SET,
            value="""{"123","ABC",12,14.567,True,False}""",
        )

    def test_str(self):
        sample_str = GlobalConstant.objects.get_constant_value("sample_str")
        self.assertEqual(sample_str, "abc")
        self.assertEqual(isinstance(sample_str, str), True)

    def test_int(self):
        sample_int = GlobalConstant.objects.get_constant_value("sample_int")
        self.assertEqual(sample_int, 123)
        self.assertEqual(isinstance(sample_int, int), True)

    def test_float(self):
        sample_float = GlobalConstant.objects.get_constant_value("sample_float")
        self.assertEqual(sample_float, 123.345)
        self.assertEqual(isinstance(sample_float, float), True)

    def test_bool(self):
        sample_bool = GlobalConstant.objects.get_constant_value("sample_bool")
        self.assertEqual(sample_bool, True)
        self.assertEqual(isinstance(sample_bool, bool), True)

    def test_dict(self):
        sample_dict = GlobalConstant.objects.get_constant_value("sample_dict")
        self.assertEqual(sample_dict, {"abc": "def", 1: "234", "4": {"a": "b"}})
        self.assertEqual(isinstance(sample_dict, dict), True)

    def test_list(self):
        sample_list = GlobalConstant.objects.get_constant_value("sample_list")
        self.assertEqual(sample_list, ["123", "ABC", 12, 14.567, True, False])
        self.assertEqual(isinstance(sample_list, list), True)

    def test_tuple(self):
        sample_tuple = GlobalConstant.objects.get_constant_value("sample_tuple")
        self.assertEqual(sample_tuple, ("123", "ABC", 12, 14.567, True, False))
        self.assertEqual(isinstance(sample_tuple, tuple), True)

    def test_set(self):
        sample_set = GlobalConstant.objects.get_constant_value("sample_set")
        self.assertEqual(sample_set, {"123", "ABC", 12, 14.567, True, False})
        self.assertEqual(isinstance(sample_set, set), True)



from setuptools import setup  # type: ignore

with open("README.md") as f:
    long_description = f.read()


setup(
    name="django_constant",
    version="1.0.0",
    description=(
        "This is a Django app designed to store global constants with different data types. It provides a central location to manage and access configuration values that can be easily reused throughout your Django project."
    ),
    long_description=long_description,
    long_description_content_type="text/markdown",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
        "Framework :: Django",
        "Framework :: Django :: 3.2",
    ],
    url="https://github.com/yashmarathe21/django-constants",
    author="Yash Marathe",
    author_email="yashmarathe21@gmail.com",
    keywords="django python constants",
    license="MIT",
    python_requires=">=3.6",
    packages=["django_constants", "django_constants/migrations/"],
    install_requires=["Django>=3.2"],
)
